@@ -1077,6 +1077,13 @@ set_locale (GnomeDesktopItem *item, const char *key,
 		char *full = g_strdup_printf ("%s[%s]", key, locale);
 		set (item, full, value);
 		g_free (full);
+
+		/* add the locale to the list of languages if it wasn't there
+		 * before */
+		if (g_list_find_custom (item->languages, locale,
+					(GCompareFunc)strcmp) == NULL)
+			item->languages = g_list_prepend (item->languages,
+							  g_strdup (locale));
 	}
 }
 
@@ -1921,8 +1928,10 @@ ditem_execute (const GnomeDesktopItem *item,
 			
 			if (item->launch_time > 0)
 				launch_time = item->launch_time;
+			else if (screen != NULL)
+				launch_time = gdk_x11_display_get_user_time (gdk_screen_get_display (screen));
 			else
-				launch_time = gtk_get_current_event_time ();
+				launch_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
 
 			sn_launcher_context_initiate (sn_context,
 						      g_get_prgname () ? g_get_prgname () : "unknown",
@@ -2037,25 +2046,26 @@ gnome_desktop_item_launch_on_screen_with_env (
 	/* This is a URL, so launch it as a url */
 	if (item->type == GNOME_DESKTOP_ITEM_TYPE_LINK) {
 		const char *url;
+		char       *free_url;
+		gboolean    retval;
+
 		url = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_URL);
-		if (url && url[0] != '\0') {
-			if (gnome_url_show (url, error))
-				return 0;
-			else
-				return -1;
+		if (url && url[0] != '\0')
+			free_url = gnome_vfs_make_uri_canonical (url);
 		/* Gnome panel used to put this in Exec */
-		} else if (exec && exec[0] != '\0') {
-			if (gnome_url_show (exec, error))
-				return 0;
-			else
-				return -1;
-		} else {
+		else if (exec && exec[0] != '\0')
+			free_url = gnome_vfs_make_uri_canonical (exec);
+		else {
 			g_set_error (error,
 				     GNOME_DESKTOP_ITEM_ERROR,
 				     GNOME_DESKTOP_ITEM_ERROR_NO_URL,
 				     _("No URL to launch"));
 			return -1;
 		}
+
+		retval = gnome_url_show (free_url, error);
+		g_free (free_url);
+		return retval ? 0 : -1;
 	}
 
 	/* check the type, if there is one set */
@@ -2799,8 +2809,6 @@ gnome_desktop_item_get_location (const GnomeDesktopItem *item)
  * @location: A uri string specifying the file location of this particular item.
  *
  * Set's the 'location' uri of this item.
- *
- * Returns:
  */
 void
 gnome_desktop_item_set_location (GnomeDesktopItem *item, const char *location)
@@ -2847,8 +2855,6 @@ gnome_desktop_item_set_location (GnomeDesktopItem *item, const char *location)
  * @file: A local filename specifying the file location of this particular item.
  *
  * Set's the 'location' uri of this item to the given @file.
- *
- * Returns:
  */
 void
 gnome_desktop_item_set_location_file (GnomeDesktopItem *item, const char *file)
